#!/usr/bin/env python3
################################################################################
#
# get_index_output_auto_ssp_elines_rnd_rss_outFIT3D.py
# ----------------------
# Perl script: get_index_output_auto_ssp_elines_rnd_rss_outFIT3D.pl
#
################################################################################

# System imports
import io
import sys
import numpy as np
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import get_index
from pyFIT3D.common.constants import __indices__
from pyFIT3D.common.io import get_data_from_fits, get_wave_from_header

def parse_value(x):
    try:
        _x = eval(x)
    except (NameError, SyntaxError):
        _x = str(x)
    return _x

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_fits', 'n_sim', 'input_auto_ssp_rss_output']
    __optional__ = ['plot', 'output_file']
    __def_optional = {'plot': 0, 'output_file': 'none'}
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_fits': str, 'n_sim': eval, 'input_auto_ssp_rss_output': str}
    __conv_func_optional__ = {'plot': parse_value, 'output_file': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' output_auto_ssp_elines_several.RSS.cube.fits NSIM auto_ssp.rss.out [PLOT] [OUTPUT_FILE]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

def main(args):
    output_file = args.output_file
    clf = False
    if output_file is not None:
        if isinstance(output_file, io.TextIOWrapper):
            f = output_file
        else:
            clf = True
            f = open(output_file, 'w')
    else:
        f = sys.stdout
    redshift__s = np.loadtxt(args.input_auto_ssp_rss_output, usecols=7, delimiter=',')
    flux__tsw, h = get_data_from_fits(args.input_fits, header=True)
    flux_ssp__sw = flux__tsw[0] - flux__tsw[3]
    res__sw = flux__tsw[4]
    plot = 0 if args.plot is None else args.plot
    indices = get_index(get_wave_from_header(h, 1),
                        flux_ssp__sw, res__sw,
                        redshift__s, args.n_sim, plot)
    names__i = list(__indices__.keys())
    for i_s in range(res__sw.shape[0]):
        for i in range(len(names__i)):
            name = names__i[i]
            print(f'{name}', end=' ', file=f)
            EW_now = indices[name]['EW'][i_s]
            s_EW_now = indices[name]['sigma_EW'][i_s]
            print(f' {EW_now} {s_EW_now}', end=' ', file=f)
        SN = indices['SN'][i_s]
        e_SN = indices['e_SN'][i_s]
        print(f'SN  {SN} {e_SN}', file=f)
    if clf:
        f.close()

if __name__ == '__main__':
    # pa stands for parsed arguments
    main(ReadArgumentsLocal())
    # pa =
    # get_index_output_auto_ssp_elines_rnd_rss_outFIT3D(pa.input_fits, pa.n_sim,
    #                                                   pa.input_auto_ssp_rss_output, pa.plot)



#!/usr/bin/env python3
import sys
import numpy as np
from os.path import basename, isfile
from pyFIT3D.common.io import ReadArguments, readFileArgumentParser
from pyFIT3D.common.auto_ssp_tools import auto_ssp_spec
from pyFIT3D.common.constants import __selected_R_V__, __selected_extlaw__
# from auto_ssp_elines_rnd_sigma_inst import ReadArgumentsLocal

# def debug_var(debug_mode=False, **kwargs):
#     pref = kwargs.pop('pref', '>>>')
#     verbose_level = kwargs.pop('verbose_level', 0)
#     pref = '\t' * verbose_level + pref
#     if debug_mode:
#         for kw, vw in kwargs.items():
#             if isinstance(vw, dict):
#                 print('%s') % pref, kw
#                 for k, v in vw.items():
#                     print('\t%s' % pref, k, ':\t', v)
#             else:
#                 print('%s' % pref, '%s:\t' % kw, vw)
#
# def parse_old_arguments(append_args=None):
#     # create new scheme
#     oa = ReadArgumentsLocal()
#     new_argv = f'{sys.argv[0]} --error_file {oa.error_file} --nl_ssp_models_file {oa.ssp_nl_fit_file}'
#     new_argv += f' --out_file {oa.out_file} --mask_list {oa.mask_list} --elines_mask_file {oa.elines_mask_file}'
#     new_argv += f' --instrumental_dispersion {oa.sigma_inst} --min_flux_plot {oa.min} --max_flux_plot {oa.max}'
#     new_argv += f' --wl_range {oa.w_min} {oa.w_max} --nl_wl_range {oa.nl_w_min} {oa.nl_w_max} --plot {oa.plot}'
#     new_argv += f' --redshift_set {oa.input_redshift} {oa.delta_redshift} {oa.min_redshift} {oa.max_redshift}'
#     new_argv += f' --losvd_set {oa.input_sigma} {oa.delta_sigma} {oa.min_sigma} {oa.max_sigma}'
#     new_argv += f' --AV_set {oa.input_AV} {oa.delta_AV} {oa.min_AV} {oa.max_AV}'
#     if append_args is not None:
#         for k, v in append_args.items():
#             new_argv += f' {k} {v}'
#     new_argv += f' {oa.spec_file} {oa.ssp_file} {oa.config_file}'
#     return new_argv.split()

def parse_arguments(default_args_file=None):
    """
    Parse the command line args.

    With fromfile_pidxrefix_chars=@ we can read and parse command line args
    inside a file with @file.txt.
    default args inside default_args_file
    """
    default_args = {
        'error_file': None, 'error_flux_variance': False, 'single_ssp': False,
        'nl_ssp_models_file': None, 'out_file': 'auto_ssp.out',
        'mask_list': None, 'elines_mask_file': None,
        'plot': 0,  # TODO: 'plot': False, 'plot_to_file': '',
        'min_flux_plot': None, 'max_flux_plot': None,
        'instrumental_dispersion': None, 'wl_range': None, 'nl_wl_range': None,
        'redshift_set': None, 'losvd_set': None, 'AV_set': None,
        'fit_sigma_nnls': False, 'no_eml_fit': False, 'sigma_in_AA': False,
        'losvd_rnd_medres_merit': False,
        'extlaw': __selected_extlaw__, 'R_V': __selected_R_V__,
    }

    help_args = {
        'spec_file': 'A file containing the spectrum information (columns should be: ID WAVELENGTH FLUX [ERROR_FLUX])',
        'error_file': 'An optional file with the error (std-dev) of the obseved spectrum',
        'ssp_models_file': 'Stellar population synthesis models. For more information about ssp_models file see README.txt.',
        'nl_ssp_models_file': 'Stellar population models used for the fit of the non-linear parameters (redshift, sigma and AV). If not used, the program uses SSP_MODELS_FILE by default',
        'config_file': 'Auto-SSP config file',
        'mask_list': 'Rest-frame wavelength ranges to be masked',
        'elines_mask_file': 'File with rest-frame emission lines central wavelength masked during the non-linear fit and stellar population synthesis',
        'out_file': 'The output parameters file. Also will set the filename prefix for other produced files',
        'instrumental_dispersion': 'The instrumental dispersion of the observed spectra. Unit in Angstroms.',
        'min_flux_plot': 'Used when plot != 0. Set the min flux used in plots',
        'max_flux_plot': 'Used when plot != 0. Set the max flux used in plots',
        'wl_range': 'Stellar population synthesis wavelength range',
        'nl_wl_range': 'Non-linear parameters fit wavelength range (only used in redshift and sigma fit)',
        'plot': '0 = no plot | 1 = plot in screen | 2 = plot to file',
        # TODO: 'plot_to_file': 'All plots to files.'
        'redshift_set': 'Redshift fit setup. The inputs are GUESS DELTA MIN MAX',
        'losvd_set': 'Line-of-sight velocity dispersion fit setup. The inputs are in the same way as the redshift fit setup. The used unit is Angstrom when --sigma_in_AA active, otherwise in km/s.',
        'AV_set': 'Dust extinction parameter fit setup. The inputs are in the same way as the redshift fit setup. The used unit is mag.',
        'no_eml_fit': 'Do not perform the emission lines fit',
        'sigma_in_AA': 'Set the line-of-sight velocity dispersion unit as Angstroms',
        'error_flux_variance': 'Treat the optional ERROR_FLUX column of spec_file as variance',
        'force_seed': 'Force the seed of the random number generator',
        'fit_sigma_nnls': 'Fit sigma using non-negative least squares (faster but less precise)',
        'single_ssp': 'Run the stellar population synthesis looking for a single SSP as an answer',
        'losvd_rnd_medres_merit': 'When using the default losvd fit, uses the median of the residual spectrum (obs - mod) instead the Chi-squared as the merit function',
        'R_V': 'R_V extinction factor: R_V=A_V/E(B-V)',
        'extlaw': 'Extinction law used. Cardelli, Clayton & Mathis (CCM) or Calzetti (CAL)',
    }

    parser = readFileArgumentParser(fromfile_prefix_chars='@')

    parser.add_argument('--error_file', metavar='FILE', type=str, default=default_args['error_file'], help=help_args['error_file'])
    parser.add_argument('-n','--nl_ssp_models_file', metavar='FILE', type=str, default=default_args['nl_ssp_models_file'], help=help_args['nl_ssp_models_file'])
    parser.add_argument('--mask_list', metavar='FILE', type=str, default=default_args['mask_list'], help=help_args['mask_list'])
    parser.add_argument('--elines_mask_file', metavar='FILE', type=str, default=default_args['elines_mask_file'], help=help_args['elines_mask_file'])
    parser.add_argument('--out_file', '-o', metavar='FILE', type=str, default=default_args['out_file'], help=help_args['out_file'])
    parser.add_argument('--instrumental_dispersion', '-d', metavar='FLOAT', type=float, default=default_args['instrumental_dispersion'], help=help_args['instrumental_dispersion'])
    parser.add_argument('--min_flux_plot', metavar='FLOAT', type=float, default=default_args['min_flux_plot'], help=help_args['min_flux_plot'])
    parser.add_argument('--max_flux_plot', metavar='FLOAT', type=float, default=default_args['max_flux_plot'], help=help_args['max_flux_plot'])
    parser.add_argument('--wl_range', nargs=2, metavar='INT', type=int, help=help_args['wl_range'])
    parser.add_argument('--nl_wl_range', nargs=2, metavar='INT', type=int, help=help_args['nl_wl_range'])
    parser.add_argument('--force_seed', metavar='INT', help=help_args['force_seed'])
    parser.add_argument('--R_V', metavar='FLOAT', type=float, default=default_args['R_V'], help=help_args['R_V'])
    parser.add_argument('--extlaw', type=str, choices=['CCM', 'CAL'], default=default_args['extlaw'], help=help_args['extlaw'])
    # TODO: plot to file
    parser.add_argument('--plot', '-p', type=int, choices=[0, 1, 2], help=help_args['plot'])
    # parser.add_argument('--plot', '-p', action='store_true', default=default_args['plot'], help=help_args['plot'])
    # parser.add_argument('--plot_to_file', metavar='FILE_PREFIX', type=str, default=default_args['plot_to_file'], help=help_args['plot_to_file'])
    parser.add_argument('--redshift_set', '-R', type=float, nargs=4, metavar='FLOAT', help=help_args['redshift_set'])
    parser.add_argument('--losvd_set', '-S', type=float, nargs=4, metavar='FLOAT', help=help_args['losvd_set'])
    parser.add_argument('--AV_set', '-A', type=float, nargs=4, metavar='FLOAT', help=help_args['AV_set'])
    parser.add_argument('--single_ssp', action='store_true', default=default_args['single_ssp'], help=help_args['single_ssp'])
    parser.add_argument('--no_eml_fit', action='store_true', default=default_args['no_eml_fit'], help=help_args['no_eml_fit'])
    parser.add_argument('--sigma_in_AA', action='store_true', default=default_args['sigma_in_AA'], help=help_args['sigma_in_AA'])
    parser.add_argument('--error_flux_variance', action='store_true', default=default_args['error_flux_variance'], help=help_args['error_flux_variance'])
    parser.add_argument('--fit_sigma_nnls', action='store_true', default=default_args['fit_sigma_nnls'], help=help_args['fit_sigma_nnls'])
    parser.add_argument('--losvd_rnd_medres_merit', action='store_true', default=default_args['losvd_rnd_medres_merit'], help=help_args['losvd_rnd_medres_merit'])

    # positional arguments
    parser.add_argument('spec_file', metavar='SPEC_FILE', type=str, help=help_args['spec_file'])
    parser.add_argument('ssp_models_file', metavar='SSP_MODELS_FILE', type=str, help=help_args['ssp_models_file'])
    parser.add_argument('config_file', metavar='CONFIG_FILE', type=str, help=help_args['config_file'])

    parser.add_argument('--verbose', '-v', action='count', default=0)

    args_list = sys.argv[1:]
    # if exists file default.args, load default args
    if default_args_file is not None and isfile(default_args_file):
        args_list.insert(0, '@%s' % default_args_file)
    args = parser.parse_args(args=args_list)
    # TREAT ARGUMENTS HERE
    print(f'ef={args.error_file} sp={args.spec_file}')
    print(f'sspnlf={args.nl_ssp_models_file} sspf={args.ssp_models_file}')
    return args

if __name__ == '__main__':
    # already in new format
    # if len(sys.argv) > 1 and (not (sys.argv[1].startswith('-') or sys.argv[1].startswith('@'))):
    #     sys.argv = parse_old_arguments()
    #     for i, arg in enumerate(sys.argv):
    #         print(i, arg)
    pa = parse_arguments()
    # debug_var(True, parsed_arguments=pa)
    # sys.exit()

    # pa = ReadArgumentsLocal()
    auto_ssp_spec(
        spec_file=pa.spec_file,
        ssp_models_file=pa.ssp_models_file,
        config_file=pa.config_file,
        out_file=pa.out_file,

        error_file=pa.error_file,
        nl_ssp_models_file=pa.nl_ssp_models_file,
        mask_list=pa.mask_list, elines_mask_file=pa.elines_mask_file,
        instrumental_dispersion=pa.instrumental_dispersion,
        wl_range=pa.wl_range, nl_wl_range=pa.nl_wl_range,
        plot=pa.plot, min=pa.min_flux_plot, max=pa.max_flux_plot,

        redshift_set=pa.redshift_set, losvd_set=pa.losvd_set, AV_set=pa.AV_set,

        fit_gas=(not pa.no_eml_fit),
        variance_error_column=pa.error_flux_variance,
        fit_sigma_rnd=(not pa.fit_sigma_nnls),
        seed=pa.force_seed,
        single_ssp=pa.single_ssp,
        losvd_rnd_medres_merit=pa.losvd_rnd_medres_merit,

        verbose=pa.verbose,

        # TODO: this option need to be implemented in StPopSynt class
        losvd_in_AA=pa.sigma_in_AA,
        R_V=pa.R_V, extlaw=pa.extlaw,
    )



#### System imports
import os
import io
import sys
import time
import itertools
import subprocess
import numpy as np
from os import getcwd
import argparse as ap
from astropy.io import fits
from datetime import datetime
from copy import deepcopy as copy
from os.path import basename, isfile, join, abspath

# import matplotlib.pyplot as plt
# import seaborn as sns
# sns.set(context="talk", style="ticks", palette="colorblind", color_codes=True)

class readFileArgumentParser(ap.ArgumentParser):
    def __init__(self, *args, **kwargs):
        super(readFileArgumentParser, self).__init__(*args, **kwargs)

    def convert_arg_line_to_args(self, line):
        for arg in line.split():
            if not arg.strip():
                continue
            if arg[0] == '#':
                break
            yield arg

#### Local imports
from .constants import __n_models_params__, __FWHM_to_sigma__, __version__

class ReadArguments(object):
    """
    Argument parser for the FIT3D echo-system
    """
    # class static configuration:
    # arguments names and conversion string to number functions

    def __init__(self, args_list=None, verbose=False):
        self.verbose = verbose
        """Initialize the argument list received and other helper variables"""
        self.args_list = args_list
        # if no list is in args_list, reads sys.argv, without the script pathname (first position of the list)
        if self.args_list is None:
            self.args_list = sys.argv[1:]
        self.N = len(self.args_list)
        # Error: less args than needed
        if (self.N < len(self.__mandatory__)) | (self.N > self.__N_tot_args__):
            print(self.__usage_msg__)
            sys.exit()
        # attributes values to arguments
        for i in range(self.N):
            arg_name = self.__arg_names__[i]
            # arguments values passed by the user
            arg_value = self.args_list[i]
            print_verbose(f'i) arg_name --------- {arg_name}', verbose=self.verbose)
            print_verbose(f'i) arg_value -------- {arg_value}', verbose=self.verbose)
            # None values
            if arg_value not in ['none', 'None']:
                # print(f'{arg_name}:{arg_value}')
                _cf = self.__conv_func__.get(arg_name, eval)
                print_verbose(f'i) conversion_func -- {_cf}', verbose=self.verbose)
                setattr(self, arg_name, _cf(arg_value))
        # attributes a default value to arguments that were not defined the by user
        i += 1
        while i < self.__N_tot_args__:
            arg_name = self.__arg_names__[i]
            try:
                arg_value = self.__def_optional__.get(arg_name, None)
            except AttributeError:
                arg_value = None
            # print(f'{arg_name}:{arg_value}')
            print_verbose(f'ii) arg_name --------- {arg_name}', verbose=self.verbose)
            print_verbose(f'ii) arg_value -------- {arg_value}', verbose=self.verbose)
            if arg_value is not None:
                if not isinstance(arg_value, str):
                    _cf = lambda x: x
                else:
                    try:
                        _cf = self.__conv_func__.get(arg_name, eval)
                    except AttributeError:
                        _cf = eval
                print_verbose(f'ii) conversion_func -- {_cf}', verbose=self.verbose)
                setattr(self, arg_name, _cf(arg_value))
            i += 1

    def __getattr__(self, attr):
        """Return None if some inexistent argument is accessed"""
        r = self.__dict__.get(attr, None)
        return r

def read_first_line(filename):
    """Return the first record of the given filename

    Parameters
    ----------
    filename : str
        The name of the file for which to get the first record

    Returns
    -------
    str
        The first not commented record found in the given filename
    """
    with open(filename) as f:
        l = f.readline()
        # jump comments
        while l.startswith('#'):
            l = f.readline()
    return l

def print_time(print_seed=True, time_ini=None, get_time_only=False):
    """ Return the local timestamp

    Parameters
    ----------
    print_seed: boolean
        Whether to print or not a formatted version of the local time

    Returns
    -------
    int
        The rounded current timestamp
    """
    time_ini = time.time() if time_ini is None else time_ini
    tepoch = int(time_ini)
    seedstr = ''
    if print_seed:
        seedstr = f' (random number generator seed: {tepoch})'
    if not get_time_only:
        print(f"# TIME {datetime.fromtimestamp(tepoch)}{seedstr}\n")
    return tepoch

# this assumes that the number of columns does not vary within the file
def get_num_col_file(filename, sep=None, maxsplit=-1):
    """ Reads the number of columns of file using `str.split`.

    Parameters
    ----------
    filename : str
        Filename of the inspected file.
    sep : None or char, optional
        If `sep` is given uses as the column delimiter. Default is None.
    maxsplit : int, optional
        If maxsplit is given, at most maxsplit splits are done (thus, the list
        will have at most maxsplit+1 elements). If maxsplit is not specified or
        -1, then there is no limit on the number of splits (all possible splits
        are made).

    Returns
    -------
    int
        Number of columns in `filename` separated by `sep`.

    """
    l = read_first_line(filename)
    return len(l.split(sep=sep, maxsplit=maxsplit))

def read_spectra(filename, f_error=None, variance_column=True):
    """Return the wavelength, and the flux and error spectra from a given filename

    If the given filename contains only 2 columns or less an error will be raised.
    If there are only 3 columns, the error will be computed as follow:

      0.002 * abs(flux)

    Parameters
    ----------
    filename : str
        Filename of the spectra.

    f_error : function, optional
        Function that defines the error when it is not present in the spectrum file.
        It defaults to 0.002 * abs(flux).

    variance_column : bool, optional
        When True treats the error column as variance and takes the square-root of the
        values.

    Returns
    -------
    {wave, flux, eflux} : array like
       The wavelength, and the flux and the error flux stored in the given file.
    """
    if f_error is None:
        f_error = lambda x: 0.002*np.abs(x)

    N_col_spec_file = get_num_col_file(filename)
    if N_col_spec_file < 2:
        raise IOError(f'read_spectra: the file "{filename}" has missing data (columns: {N_col_spec_file}), expected at least 3')
    if N_col_spec_file == 4:
        wave, flux, eflux = np.loadtxt(filename, unpack=True, usecols=(1,2,3))
        eflux = np.abs(eflux)
        if variance_column:
            eflux = np.sqrt(eflux)
    else:  # no error avaible.
        usecols = (1, 2)
        if N_col_spec_file == 2: # missing index column in file
            usecols = (0, 1)
        wave, flux = np.loadtxt(filename, unpack=True, usecols=usecols)
        eflux = f_error(flux)
    return wave, flux, eflux

def output_spectra(w, v, filename):
    """
    A wrapper to np.savetxt() creating a column with an ID.
    """

    _out = list(zip(list(range(1, w.size+1)), w, *v))
    # np.savetxt(filename, _out, fmt='%s')
    np.savetxt(filename, _out, fmt=['%d'] + (len(v) + 1)*['%.18g'])

def probe_method_time_consumption(**kwargs):
    """Function created as a wrapper to cProfile Python Profilers, a set of statistics that describes how often and how long various parts of the program (or any call of method/functions) executed.

    Example
    -------

    .. code-block:: python

        def f_sum_xy_ntimes(x, y, n=100):
            for i in range(n):
                s = x + y
            return s

        a = np.random.normal(10, 0.3, 100)
        b = np.random.normal(3, 0.1, 100)
        a_plus_b = probe_method_time_consumption(f=f_sum_xy_ntimes, f_args=dict(x=a, y=b, n=10000))

        #         2 function calls in 0.011 seconds
        #
        #   Ordered by: cumulative time
        #
        #   ncalls  tottime  percall  cumtime  percall filename:lineno(function)
        #        1    0.011    0.011    0.011    0.011 <ipython-input-32-389c091e3289>:1(f_sum_xy_ntimes)
        #        1    0.000    0.000    0.000    0.000 {method 'disable' of '_lsprof.Profiler' objects}
        # ...

    Parameters
    ----------
    f : function
        Function that will be probed.

    f_args : dict
        A dictionary with the arguments for `f` call.

    Returns
    -------
    ret_f
        The return from `f`.
    """
    import cProfile, pstats, io
    f = kwargs.get('f')
    f_args = kwargs.get('f_args')
    pr = cProfile.Profile()
    pr.enable()
    f_ret = f(**f_args)
    pr.disable()
    s = io.StringIO()
    sortby = 'cumulative'
    ps = pstats.Stats(pr, stream=s).sort_stats(sortby)
    ps.print_stats()
    print(s.getvalue())
    return f_ret

def remove_isfile(filename):
    if os.path.isfile(filename):
        os.remove(filename)

# EL: deprecated
def clean_preview_results(args):
    """Remove output files from older runs

    Parameters
    ----------
        args : class `ReadArguments`
    """

    if os.path.isfile(args.out_file):
        os.remove(args.out_file)
    if os.path.isfile(args.out_file_elines):
        os.remove(args.out_file_elines)
    if os.path.isfile(args.out_file_single):
        os.remove(args.out_file_single)
    if os.path.isfile(args.out_file_coeffs):
        os.remove(args.out_file_coeffs)
    if os.path.isfile(args.out_file_fit):
        os.remove(args.out_file_fit)

def clean_preview_results_files(out_file, out_file_elines, out_file_single,
                                out_file_coeffs, out_file_fit):
    """Remove output files from older runs

    Parameters
    ----------
        args : class `ReadArguments`
    """
    remove_isfile(out_file)
    remove_isfile(out_file_elines)
    remove_isfile(out_file_single)
    remove_isfile(out_file_coeffs)
    remove_isfile(out_file_fit)

def trim_waves(wave, wave_range):
    """Return a mask for the trimmed version of the wavelength range

    For a given observed wavelength for which it is intended to extract
    physical information, this function returns a boolean selection of
    the wavelength range to consider during the fitting. The selection
    is computd such that:

    wave_range[0] <= wave <= wave_range[1]

    In the case where wave_range is None or wave_range[0] = wave_range[1],
    returns the entire wavelength range.

    Parameters
    ----------
    wave: array like
        A one-dimensional array of the observed wavelength
    """
    if (wave_range is None) or (wave_range[0] == wave_range[1]):
       mask = np.ones(wave.shape, dtype='bool')
    else:
       mask = (wave >= wave_range[0]) & (wave <= wave_range[1])
    return mask

def sel_waves(masks, wave):
    """Return the mask for the given wavelength array

    For a given observed wavelength for which it is intended to extract
    physical information, this function returns a boolean selection of wavelength
    ranges to consider during the fitting. The selections are computed such that for
    any i-mask:

    masks[i, 0] <= wave <= masks[i, 1]

    Parameters
    ----------
    masks: array like
        A Nx2 array containing the masks for the emission lines
    wave: array like
        A one-dimensional array of the observed wavelength

    Returns
    -------
    array like :
        A boolean array with True for the non-masked wavelengths.

    See Also
    --------
    read_masks_file
    """
    selected_wavelengths = np.ones(wave.size, dtype='bool')
    if (masks is not None) and (len(masks) > 0):
        for left, right in masks:
            # limit to above left limit
            left_selection = wave >= left
            # limit to below right limit
            right_selection = wave <= right
            # selects all range
            range_selection = left_selection & right_selection
            # mask masked range
            selected_wavelengths[range_selection] = False
    return selected_wavelengths

def read_masks_file(filename):
    """
    Read the masks file and returns an array with the ranges to be masked.

    Parameters
    ----------
    filename : str
        Masks filename.

    Returns
    -------
    array like :
        An array that saves in each position the range to be masked.
    int :
        The number of intervals to be masked.

    Example
    -------
    A typical mask file `filename`:

    5800 6080
    5550 5610
    7100 15000

    should return:

    array([5800., 6808.],
          [5500., 5610.],
          [7100., 15000.]),
    """
    # read masks
    masks = None
    n_masks = 0
    if filename is not None:
        if isfile(filename):
            masks = np.loadtxt(filename)
            n_masks = len(masks)
        else:
            print(f'{basename(sys.argv[0])}: {filename}: mask list file not found')
    else:
        print(f'{basename(sys.argv[0])}: no mask list file')
    return masks, n_masks

def print_verbose(text, verbose=0, level=1):
    """ Print `text` if verbose is True.

    Parameters
    ----------
    text : str
        Text to be printed.
    verbose : int, bool, optional
        If `verbose` is greater than `level` prints `text`. If is True forces
        the print of `text`. Defaults to 0.
    level : int, optional
        Configures the print level of the verbosity. Default level is 1.
    """
    if (level if (0 if verbose is None else verbose) is True else verbose) >= level:
        print(text)

def plot_spectra_ax(ax, wave_list, spectra_list, title='', labels_list=None,
                    color=None, cmap=None, ylim=None, xlim=None, alpha=None, lw=None):
    """
    Print spectra provided by `wave_list` and `spectra_list`.

    Parameters
    ----------
    ax : matplotlib.axis
        Axis to plot spectra.
    wave_list : list
        List of `spectra_list` wavelengths.
    spectra_list : list
        List of spectra.
    title: str, optional
        Title to the axis. Default value is ''.
    labels_list : list or None
        List of labels, if None no labels are shown.
    color: str, list or None
        A color, a list of colors or None. If None automatically generates the colors
        for `spectra_list` by using the `cmap` colormap.
    cmap: str
        The colormap to generate the colors for spectra if `color` is None.
        Default colormap is viridis.
    xlim: list or None
        Plot boudaries of the x-axis. Default value is None (i.e., define automatically the
        x-limits).
    ylim: list or None
        Plot boudaries of the y-axis. Default value is None (i.e., define automatically the
        y-limits).
    alpha: float, list or None
        An alpha, a list of alphas or None. If None automatically uses alpha=1.
    lw: float, list or None
        A line width (lw), a list of lw or None. If None automatically uses defaul matplotlib lw.
    """
    if 'seaborn' not in sys.modules:
        import seaborn as sns
    else:
        sns = sys.modules['seaborn']

    N_wave = len(wave_list)
    N_spec = len(spectra_list)
    if (N_spec > 1) and (N_wave != N_spec):
        wave_list = [wave_list[0]]*N_spec
    ax.set_title(title)
    if alpha is None:
        alpha = [1]*N_spec
    if not isinstance(alpha, list):
        alpha = [alpha]*N_spec
    alpha_list = alpha
    if lw is None:
        lw = [1]*N_spec
    if not isinstance(lw, list):
        lw = [lw]*N_spec
    lw_list = lw
    if color is None:
        # cm = plt.cm.get_cmap(cmap if cmap is not None else "colorblind")
        # colors_list = [cm(i/N_spec) for i in range(N_spec)]
        colors_list = sns.color_palette(cmap if cmap is not None else "colorblind", n_colors=N_spec)
    else:
        if isinstance(color, str):
            colors_list = [color]*N_spec
        else:
            if len(color) < N_spec:
                colors_list = [color[0]]*N_spec
            else:
                colors_list = color
    legend = True
    if labels_list is None:
        legend = False
        labels_list = ['']*N_spec
    for w, f, c, l, a, lw in zip(wave_list, spectra_list, colors_list, labels_list, alpha_list, lw_list):
        ax.plot(w, f, color=c, label=l, alpha=a, lw=lw)
    # it is very expensive in time to call `ax.legend` if no label is set.
    if legend:
        ax.legend(loc=2, frameon=False)
    if xlim is not None:
        ax.set_xlim(xlim)
    if ylim is not None:
        ax.set_ylim(ylim)

def get_data_from_fits(filename, extension=0, header=None, return_n_extensions=False):
    """ Retrieve the data from the HDU `extension` in a FITS file.
    If `header` is True, also, retrieves the header from the HDU
    `extension`.
    if `filename` is not a existent file or `astropy.io.fits.open(filename)`
    raises an `OSError`, returns the `filename`. if header is True,
    returns None.

    Parameters
    ----------
    filename: str
        FITS file name.

    extension: int
        The retrieved HDU extension. Default is 0.

    header: bool or None
        Also retrieves the HDU `extension` header.

    return_n_extensions: bool or None
        Also returns the number of `extensions` of the FITS file.

    Returns
    -------
    array like:
        The data from HDU `extension`.

    astropy.io.fits.header.Header:
        If `header` is True also returns the header from HDU `extension`.

    int:
        If `return_n_extensions` is True also returns the number of extensions.
    """
    data = filename
    h = None
    n_ext = 0
    if filename is not None and os.path.isfile(filename):
        try:
            with fits.open(filename) as t:
                hdu = t[extension]
                hdu.verify('silentfix')
                data = copy(hdu.data)
                h = copy(hdu.header)
                n_ext = len(t)
        except OSError:
            print(f'{sys.argv[0]}: {filename}: not a valid FITS file.')
            pass
    else:
        print(f'{sys.argv[0]}: {filename}: not a valid file. Returning the input value.')
    if header:
        if return_n_extensions:
            return data, h, n_ext
        else:
            return data, h
    else:
        if return_n_extensions:
            return data, n_ext
        else:
            return data

def get_wave_from_header(header, wave_axis=None):
    '''
    Generates a wavelength array using `header`, a :class:`astropy.io.fits.header.Header`
    instance, at axis `wave_axis`.

    wavelengths = CRVAL + CDELT*([0, 1, ..., NAXIS] + 1 - CRPIX)

    Parameters
    ----------
    header : :class:`astropy.io.fits.header.Header`
        FITS header with spectral data.

    wave_axis : int, optional
        The axis where the wavelength information is stored in `header`,
        (CRVAL, CDELT, NAXIS, CRPIX).
        Defaults to 1.

    Returns
    -------
    array like
        Wavelengths array.

        wavelengths = CRVAL + CDELT*([0, 1, ..., NAXIS] + 1 - CRPIX)
    '''
    if wave_axis is None:
        wave_axis = 1
    h = header
    crval = h[f'CRVAL{wave_axis}']
    cdelt = h[f'CDELT{wave_axis}']
    naxis = h[f'NAXIS{wave_axis}']
    crpix = h[f'CRPIX{wave_axis}']
    if not cdelt:
        cdelt = 1
    return crval + cdelt*(np.arange(naxis) + 1 - crpix)

def array_to_fits(filename, arr, new_card=False, header=None, overwrite=False, sort_dict_header=True):
    hdul = fits.HDUList()
    if new_card:
        hdul.append(fits.PrimaryHDU())
        hdul.append(fits.ImageHDU(data=arr))
    else:
        hdul.append(fits.PrimaryHDU(arr))
    hdu = hdul[0]
    # XXX version should be a global parameter
    if header is not None:
        if isinstance(header, dict):
            if sort_dict_header:
                header = dict(sorted(header.items()))
            for k, v in header.items():
                hdu.header.set(k, value=v)
        else:
            hdu.header = header
    hdu.header.set('PIPELINE', value=__version__)
    now = time.time()
    dt_now = datetime.fromtimestamp(now)
    hdu.header.set('UNIXTIME', value=int(now), comment=f'{dt_now}')
    hdul.writeto(filename, overwrite=overwrite)

def call_cmnd(cmnd, logfile=None, verbose=True, logfilemode='a', timeout=None):
    close = False
    if isinstance(logfile, str):
        close = True
        lfd = open(logfile, mode=logfilemode)
    elif isinstance(logfile, io.TextIOWrapper):
        lfd = logfile
    else:
        lfd = sys.stdout
    if verbose:
        print(cmnd, file=lfd)
    try:
        output = subprocess.check_output(cmnd, stderr=subprocess.STDOUT,
                                         shell=True, timeout=timeout, universal_newlines=True)
    except subprocess.CalledProcessError as exc:
        print("Status : FAIL", exc.returncode, exc.output, file=lfd)
    else:
        print("Output: \n{}\n".format(output), file=lfd)

def print_done(time_ini, message=None):
    message = 'DONE!' if message is None else message
    time_end = print_time(print_seed=False, get_time_only=True)
    time_spent = time_end - time_ini
    print(f'# {message} - time spent: {time_spent} s\n####\n', flush=True)

def print_block_init(message, print_seed=False):
    print(f'####\n# {message}\n#', flush=True)
    return print_time(print_seed=print_seed)

def write_img_header(input_fits, keyword, value,
                     comment=None, before=None, after=None, output=None, overwrite=True):
    if output is None:
        output = input_fits
        overwrite = True
    t, h = get_data_from_fits(input_fits, header=True)
    if isinstance(keyword, list):
        for i, k in enumerate(keyword):
            if value is None:
                v = None
            else:
                v = value if not isinstance(value, list) else value[i]
            if comment is None:
                c = None
            else:
                c = comment if not isinstance(comment, list) else comment[i]
            if before is None:
                b = None
            else:
                b = before if not isinstance(before, list) else before[i]
            if after is None:
                a = None
            else:
                a = after if not isinstance(after, list) else after[i]
            h.set(k, value=v, comment=c, before=b, after=a)
    else:
        # set new value to header
        h.set(keyword, value=value, comment=comment, before=before, after=after)
    array_to_fits(output, t, header=h, overwrite=overwrite)

def create_emission_lines_mask_file_from_list(wave_list, eline_half_range=16, output_path=None, label=None):
    output_path = getcwd() if output_path is None else abspath(output_path)
    np.savetxt(join(output_path, ('autodetect.mask_elines.txt' if label is None else f'{label}.autodetect.mask_elines.txt')),
               [[int(wave_peak-eline_half_range), int(wave_peak+eline_half_range)] for wave_peak in wave_list],
               fmt='%5d')

def create_emission_lines_file_from_list(wave_list, output_path=None, label=None):
    output_path = getcwd() if output_path is None else abspath(output_path)
    np.savetxt(join(output_path, ('autodetect.emission_lines.txt' if label is None else f'{label}.autodetect.emission_lines.txt')), wave_list, fmt='%5.2f')

def create_ConfigAutoSSP_from_lists(list_chunks, list_systems_config, output_path=None, label=None):
    # TODO: ConfigAutoSSP should be created dynamically (pyFIT3D/common/auto_ssp_tools.py)
    #       This function should return the list of wavelengths and generated
    #       config filenames to the new function.
    output_path = getcwd() if output_path is None else abspath(output_path)
    ConfigAutoSSP_filename = join(output_path, ('autodetect.auto_ssp_several.config' if label is None else f'{label}.autodetect.auto_ssp_several.config'))
    n_systems = len(list_systems_config)
    eml_systems_lines = [f'{chunk} none {config_filename} {__n_models_params__} none 20 1950\n' for chunk, config_filename in zip(list_chunks, list_systems_config)]
    with open(ConfigAutoSSP_filename, 'w') as ConfigAutoSSP_file:
        ConfigAutoSSP_file.write('0.017 0.002 0.0001 0.027 10 100 0.1 0.5 3800 5500\n')
        ConfigAutoSSP_file.write('3.2  0.0    1.9    6.5\n')
        ConfigAutoSSP_file.write('0.4  0.1    0.0   2.5\n')
        ConfigAutoSSP_file.write(f'{n_systems}\n')
        ConfigAutoSSP_file.writelines(eml_systems_lines)
        ConfigAutoSSP_file.write('0.0001  1 0.00\n')
        ConfigAutoSSP_file.write('6588 6760\n')



#!/usr/bin/env python3
################################################################################
#
# radial_sum_cube_e.py
# ----------------------
# Perl script: radial_sum_cube_e.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import get_data_from_fits, array_to_fits
from pyFIT3D.common.tools import radial_sum_cube_e

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_cube_fits', 'delta_R', 'x0', 'y0', 'output_rss_fits']
    __optional__ = ['plot']
    __def_optional__ = {'plot': '0', 'badpixel_mask_extension': 3, 'badpixel_mask_value': 1}
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_cube_fits': str, 'output_rss_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' CUBE.fits Delta_R X_C Y_C OUTPUT.RSS.fits [PLOT] [BADPIXEL_MASK] [BADPIXEL_VALUE]\n'
    __usage_msg__ += 'It will use images coordinates: 0,nx 0,ny'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

def main(args):
    e_output_rss_fits = f'e_{args.output_rss_fits}'
    weight_output_rss_fits = f'weight.{args.output_rss_fits}'
    cube__wyx, h, n_ext = get_data_from_fits(args.input_cube_fits, header=True, return_n_extensions=True)
    crval = h['CRVAL3']
    cdelt = h['CDELT3']
    crpix = h['CRPIX3']
    input_mask = None
    input_error = None
    if h['EXTEND']:
        input_error = get_data_from_fits(args.input_cube_fits, extension=1)
        if (args.badpixel_mask_extension is not None) and (n_ext > 3):
            input_mask = get_data_from_fits(args.input_cube_fits, extension=args.badpixel_mask_extension)
            if args.badpixel_mask_value != 1:
                _tmp_input_mask = np.ones_like(input_mask)
                _tmp_input_mask[input_mask != args.badpixel_mask_value] = 0
                input_mask = _tmp_input_mask
    r = radial_sum_cube_e(cube__wyx, args.delta_R, args.x0, args.y0,
                          input_mask=input_mask, input_error=input_error)
    output__rw, e_output__rw, mask_output__rw = r
    h_set = {'CRVAL1': crval, 'CDELT1': cdelt, 'CRPIX1': crpix}
    array_to_fits(args.output_rss_fits, output__rw, header=h_set, overwrite=True)
    array_to_fits(e_output_rss_fits, e_output__rw, header=h_set, overwrite=True)
    array_to_fits(weight_output_rss_fits, mask_output__rw, header=h_set, overwrite=True)

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # pa =
    # radial_sum_cube_e(pa.input_cube_fits, pa.delta_R, pa.x0, pa.y0, pa.output_rss_fits, pa.plot)



#!/usr/bin/env python3
################################################################################
#
# clean_Ha_map.py
# ----------------------
# Perl script: clean_Ha_map.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import array_to_fits, get_data_from_fits
from pyFIT3D.common.tools import clean_Ha_map

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input1_fits', 'input2_fits',]
    __optional__ = ['max_vel', 'min_vel']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __def_optional__ = {'max_vel': '1e12', 'min_vel': '0'}
    __conv_func_mandatory__ = {'input1_fits': str, 'input2_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' INPUT1.FITS INPUT2.FITS [MAX_VEL] [MIN_VEL]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

def main(args):
    v__yx, h1 = get_data_from_fits(args.input1_fits, header=True)
    mask__yx, h2 = get_data_from_fits(args.input2_fits, header=True)
    v__yx, mask__yx = clean_Ha_map(v__yx, mask__yx, args.max_vel, args.min_vel)
    array_to_fits(args.input1_fits, v__yx, overwrite=True, header=h1)
    array_to_fits(args.input2_fits, mask__yx, overwrite=True, header=h2)

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # parsed_args = ReadArgumentsLocal()
    # in1_file = parsed_args.input1_fits
    # in2_file = parsed_args.input2_fits
    # max_vel = parsed_args.max_vel
    # min_vel = parsed_args.min_vel
    # clean_Ha_map(in1_file, in2_file, max_vel, min_vel)



#!/usr/bin/env python3
################################################################################
#
# imarith.py
# ----------------------
# Perl script: imarith.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import imarith

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input1_fits', 'operator', 'input2_fits', 'output_fits']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input1_fits': str, 'operator': str,
                               'input2_fits': str, 'output_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' INPUT1.FITS OPERATOR(+,-,?,*) INPUT2.FITS OUTPUT.FITS'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    parsed_args = ReadArgumentsLocal()
    in1_file = parsed_args.input1_fits
    operator = parsed_args.operator
    in2_file = parsed_args.input2_fits
    out_file = parsed_args.output_fits
    imarith(in1_file, operator, in2_file, out_file)



#!/usr/bin/env python3
################################################################################
#
# vel_eline_cube.py
# ----------------------
# Perl script: vel_eline_cube.pl
#
################################################################################

### Local imports
import sys
import itertools
import numpy as np
from os.path import basename

from pyFIT3D.common.io import ReadArguments, get_wave_from_header
from pyFIT3D.common.io import array_to_fits, get_data_from_fits, trim_waves
# from pyFIT3D.common.tools import vel_eline_cube
from pyFIT3D.common.tools import vel_eline

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['cube_fits', 'nsearch', 'imin',
                     'prefix_out','wave_ref', 'plot']
    __optional__ = ['wmin', 'wmax']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __def_optional__ = {'wmax': '0', 'wmin': '1e12'}
    __conv_func_mandatory__ = {'cube_fits': str,
                               'prefix_out': str,
                               }
    __conv_func_optional__ = {'plot': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' CUBE.fits nsearch LIMIT(% max)'
    usage_msg_tmp3 = ' PREFIX_OUT WAVE_REF DEV [WMIN WMAX]\n'
    usage_msg_tmp4 = 'OUTPUT: PREFIX_OUT.vel_map.fits\n'
    usage_msg_tmp5 = 'OUTPUT: PREFIX_OUT.mask_map.fits'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2 + usage_msg_tmp3 + \
            usage_msg_tmp4 + usage_msg_tmp5

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

def main(args):
    map_outfile = args.prefix_out + ".vel_map.fits"
    mask_outfile = args.prefix_out + ".mask_map.fits"
    data_in, h = get_data_from_fits(args.cube_fits, header=True)
    nz, ny, nx = data_in.shape
    map_out = np.zeros((ny, nx))
    mask_out = np.zeros((ny, nx))
    wave = get_wave_from_header(h, wave_axis=3)
    mask_wave = trim_waves(wave, [args.wmin, args.wmax])
    # wave = np.array([crval3 + cdelt3 * k for k in np.arange(nz)])
    # mask_wave = (wave > wmin) & (wave < wmax)
    wave = wave[mask_wave]
    # print(wave[mask_wave])
    for ixy in itertools.product(range(nx),range(ny)):
        ix, iy = ixy
        # if (ix == 26) & (iy == 44):
        flux = data_in[:, iy, ix]
        flux = flux[mask_wave]
        vel, mask, npeaks = vel_eline(flux, wave, args.nsearch, args.imin, args.wave_ref, set_first_peak=False)
        map_out[iy, ix] = vel
        mask_out[iy, ix] = mask
        print("{}/{}".format(ix, nx))
    h_set = {
        # 'NAXIS': 2,
        # 'NAXIS1': nx,
        # 'NAXIS2': ny,
        'COMMENT': 'vel_eline_cube result',
        'WMIN': args.wmin,
        'WMAX': args.wmax,
        'WREF': args.wave_ref,
        'FILENAME': map_outfile,
    }
    array_to_fits(map_outfile, map_out, header=h_set, overwrite=True)
    h_set['FILENAME'] = mask_outfile
    array_to_fits(mask_outfile, mask_out, header=h_set, overwrite=True)

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # parsed_args = ReadArgumentsLocal()
    # cube_fits = parsed_args.cube_fits
    # nsearch = parsed_args.nsearch
    # imin = parsed_args.imin
    # prefix_out = parsed_args.prefix_out
    # wave_ref = parsed_args.wave_ref
    # plot = parsed_args.plot
    # wmin = parsed_args.wmin
    # wmax = parsed_args.wmax
    # vel_eline_cube(cube_fits, nsearch, imin, prefix_out, wave_ref, plot, wmin, wmax)
    # ./vel_eline_cube.py /home/espinosa/tmp/GAS.UGC12250.cube.fits.gz 1 0.3 vel_Ha.UGC12250 6562.8 dev 6670 6750



import numpy as np
"""This module contains definition of pyFIT3D.common.constants to be used
globally across the pyFIT3D modules
"""

__version__ = 'pyPipe3D v1.1.5'

# maximum number of free parameters in the config file
__n_models_params__ = 9

_MODELS_VOIGT_PAR = {
    'central_wavelength': 0,
    'flux': 1,
    'sigma_L': 2,
    'sigma_G': 3,
    'v0': 4
}

_MODELS_ELINE_PAR = {
    'central_wavelength': 0,
    'flux': 1,
    'sigma': 2,
    'v0': 3
}

# _MODELS_POLY1D_PAR should be dynamically created
# _MODELS_POLY1D_PAR = lambda ncoeffs: {f'coeff{i}': i for i in range(ncoeffs)}
_MODELS_POLY1D_PAR = {
    'cont': 0
}

# possible EL MODELS
_EL_MODELS = {
    # 'voigt': _MODELS_VOIGT_PAR,
    'eline': _MODELS_ELINE_PAR,
    'poly1d': _MODELS_POLY1D_PAR,
    # 'poly1d': None,
}

__shift_convolve_lnwl_frac__ = 5  # a.k.a. f_fine in shift_convolve

__mask_elines_window__ = 4  # AA
# Emission Line fine search option
__ELRND_fine_search_option__ = False

__n_Monte_Carlo__ = 20

# speed of light in km/s
__c__ = 299792.458

# convert std to FWHM
__sigma_to_FWHM__ = 2*(2*np.log(2))**0.5

# convert FWHM to sigma
__FWHM_to_sigma__ = 1/__sigma_to_FWHM__

__solar_metallicity__ = 0.02
__solar_luminosity__ = 3.826e33  # erg/s

__selected_extlaw__ = 'CCM'
__selected_R_V__ = 3.1

__selected_half_range_wl_auto_ssp__ = 50
__selected_half_range_sysvel_auto_ssp__ = 100

# Cosmological parameters
__Hubble_constant__ = 71  # km/(s Mpc)
__Omega_matter__ = 0.27
__Omega_Lambda__ = 0.73

__Ha_central_wl__ = 6562.68

__indices__ = {
    'Hd':     [4083.500, 4122.250, 4041.600, 4079.750, 4128.500, 4161.000],
    'Hb':     [4847.875, 4876.625, 4827.875, 4847.875, 4876.625, 4891.625],
    'Mgb':    [5160.125, 5192.625, 5142.625, 5161.375, 5191.375, 5206.375],
    'Fe5270': [5245.650, 5285.650, 5233.150, 5248.150, 5285.650, 5318.150],
    'Fe5335': [5312.125, 5352.125, 5304.625, 5315.875, 5353.375, 5363.375],
    'D4000':  [4050.000, 4250.000, 3750.000, 3950.000, 0.000,    1.000],
    'Hdmod':  [4083.500, 4122.250, 4079,     4083,     4128.500, 4161.000],
    'Hg':     [4319.75,  4363.50,  4283.50,  4319.75,  4367.25,  4419.75]
}
_INDICES_POS = {
    'OL1': 0,
    'OL2': 1,
    'OLb1': 2,
    'OLb2': 3,
    'OLr1': 4,
    'OLr2': 5,
    'OLb': 6,
    'OLr': 7
}
for k, v in __indices__.items():
    OLb1 = __indices__[k][_INDICES_POS['OLb1']]
    OLb2 = __indices__[k][_INDICES_POS['OLb2']]
    OLr1 = __indices__[k][_INDICES_POS['OLr1']]
    OLr2 = __indices__[k][_INDICES_POS['OLr2']]
    OLb = (OLb1 + OLb2)/2
    OLr = (OLr1 + OLr2)/2
    __indices__[k].append(OLb)
    __indices__[k].append(OLr)

_SSP_OUTPUT_INDEX = {
    'chi_joint': 0,
    'age_min': 1, 'e_age_min': 2,
    'met_min': 3, 'e_met_min': 4,
    'AV_min': 5, 'e_AV_min': 6,
    'redshift': 7, 'e_redshift': 8,
    'sigma': 9, 'e_sigma': 10,
    'FLUX': 11, 'redshift2': 12,
    'med_flux': 13, 'rms': 14,
    'age_min_mass':15 , 'e_age_min_mass': 16,
    'met_min_mass':17 , 'e_met_min_mass': 18,
    'systemic_velocity': 19, 'lml': 20,
    'lmass': 21,
}

# plot constants
_plot_dpi = 300
_latex_ppi = 72.0
_latex_column_width_pt = 240.0
_latex_column_width = _latex_column_width_pt/_latex_ppi
# latex_column_width = latex_column_width_pt/latex_ppi/1.4
_latex_text_width_pt = 504.0
_latex_text_width = _latex_text_width_pt/_latex_ppi
_golden_mean = 0.5 * (1. + 5**0.5)
_figsize_default = (2*_latex_column_width, 2*_latex_column_width/_golden_mean)



#!/usr/bin/env python3
################################################################################
#
# FIT3D_output_rss_seg2cube.py
# ----------------------
# Perl script: FIT3D_output_rss_seg2cube.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import get_data_from_fits, array_to_fits
from pyFIT3D.common.tools import rss_seg2cube

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_fits', 'n_out', 'segmentation_fits', 'output_fits']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_fits': str, 'segmentation_fits': str, 'output_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' INPUT.FITS OUTPUT.FITS X_WIDTH Y_WIDTH'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

def main(args):
    rss__sw, h = get_data_from_fits(args.input_fits, header=True)
    wave_axis = 1
    crval = h[f'CRVAL{wave_axis}']
    cdelt = h[f'CDELT{wave_axis}']
    crpix = h[f'CRPIX{wave_axis}']
    seg__yx = get_data_from_fits(args.segmentation_fits)
    cube__wyx = rss_seg2cube(rss__sw[args.n_out], seg__yx)
    # write cube
    h_set = {'CRVAL3': crval, 'CDELT3': cdelt, 'CRPIX3': crpix}
    array_to_fits(args.output_fits, cube__wyx, header=h_set, overwrite=True)
    print(f'{args.output_fits} created')

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # pa stands for parsed arguments
    # FIT3D_output_rss_seg2cube(pa.input_fits, pa.n_out, pa.segmentation_fits, pa.output_fits)



#!/usr/bin/env python3
################################################################################
#
# spec_extract_cube_error.py
# ----------------------
# Perl script: spec_extract_cube_error.pl
#
################################################################################

# System imports
import sys
import numpy as np
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import get_data_from_fits, array_to_fits
from pyFIT3D.common.tools import spec_extract_cube_error

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_fits', 'segmentation_fits', 'output_rss_fits']
    __optional__ = ['output_diffuse_fits']
    __def_optional__ = {'output_diffuse_fits': 'diffuse.fits'}
    __arg_names__ = __mandatory__  + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_fits': str, 'segmentation_fits': str, 'output_rss_fits': str}
    __conv_func_optional__ = {'output_diffuse_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' INPUT.CUBE.fits SEGMENTATION.fits OUTPUT.RSS.FITS'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

def main(args):
    diffuse_output = 'diffuse.fits' if args.diffuse_output is None else args.diffuse_output
    # read cube
    _, h = get_data_from_fits(args.input_fits, header=True)
    error__wyx = get_data_from_fits(args.input_fits, extension=1)
    nw, ny, nx = error__wyx.shape
    # read segmentation
    seg__yx = get_data_from_fits(args.segmentation_fits)
    ns = seg__yx.max().astype('int')
    nys, nxs = seg__yx.shape

    if (nx > nxs) or (ny > nys):
        print(f'{basename(sys.argv[0])}: Dimensions does not match ({nx},{ny}) != ({nxs},{nys})')
        # XXX: deal function returns in all tools
        sys.exit()

    out_data__sw, inv_seg__yx, x, y, npt = spec_extract_cube_error(error__wyx, seg__yx)

    h_set = {'CRVAL1': h['CRVAL3'], 'CDELT1': h['CDELT3'], 'CRPIX1': h['CRPIX3']}
    array_to_fits(args.output_rss_fits, out_data__sw, header=h_set, overwrite=True)

    size = np.sqrt(nx**2+ny**2)/(2*ns)
    output_rss_txt = args.output_rss_fits.replace('fits', 'pt.txt')
    with open(output_rss_txt, 'w') as f:
        output_header = f'C {size} {size} 0'
        print(output_header, file=f)
        # output_header = f'(1) id\n(2) S/N\n(3) Signal\n(4) Noise'
        np.savetxt(f, list(zip(list(range(ns)), x/npt, y/npt, [1]*ns)),
                   fmt=['%d'] + 2*['%.18g'] + ['%d'], delimiter=' ')
    print(f'{args.output_rss_fits} and {output_rss_txt} created')

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # pa = ReadArgumentsLocal()
    # spec_extract_cube_error(pa.input_fits, pa.segmentation_fits, pa.output_rss_fits)



import numpy as np

def Cardelli_extlaw(l, R_V=3.1):
    """Calculates q(lambda) to the Cardelli, Clayton & Mathis 1989
    reddening law.

    Parameters
    ----------
    l : array like
        Wavelenght vector (Angstroms)

    R_V : float, optional
        Selective extinction parameter (roughly "slope").
        Default is R_V = 3.1.

    Returns
    -------
    array like
        q(lambda) of the reddedning law.
    """
    a = np.zeros(np.shape(l))
    b = np.zeros(np.shape(l))
    F_a = np.zeros(np.shape(l))
    F_b = np.zeros(np.shape(l))
    x = np.zeros(np.shape(l))
    y = np.zeros(np.shape(l))
    q = np.zeros(np.shape(l))
    for i in range(0,len(l)):
        x[i]=10000/l[i]
        y[i]=10000/l[i] - 1.82
    # Far-Ultraviolet: 8 <= x <= 10 ; 1000 -> 1250 Angs
    inter = np.bitwise_and(x >= 8, x <= 10)
    a[inter] = -1.073 - 0.628*(x[inter] - 8.) + 0.137*(x[inter] - 8.)**2 - 0.070*(x[inter] - 8.)**3
    b[inter] = 13.670 + 4.257*(x[inter] - 8.) - 0.420*(x[inter] - 8.)**2 + 0.374*(x[inter] - 8.)**3
    # Ultraviolet: 3.3 <= x <= 8 ; 1250 -> 3030 Angs
    inter =  np.bitwise_and(x >= 5.9, x < 8)
    F_a[inter] = -0.04473*(x[inter] - 5.9)**2 - 0.009779*(x[inter] - 5.9)**3
    F_b[inter] =  0.2130*(x[inter] - 5.9)**2 + 0.1207*(x[inter] - 5.9)**3
    inter =  np.bitwise_and(x >= 3.3, x < 8)
    a[inter] =  1.752 - 0.316*x[inter] - 0.104/((x[inter] - 4.67)**2 + 0.341) + F_a[inter]
    b[inter] = -3.090 + 1.825*x[inter] + 1.206/((x[inter] - 4.62)**2 + 0.263) + F_b[inter]
    # Optical/NIR: 1.1 <= x <= 3.3 ; 3030 -> 9091 Angs ;
    inter = np.bitwise_and(x >= 1.1, x < 3.3)
    a[inter] = 1.+ 0.17699*y[inter] - 0.50447*y[inter]**2 - 0.02427*y[inter]**3 + 0.72085*y[inter]**4 + 0.01979*y[inter]**5 - 0.77530*y[inter]**6 + 0.32999*y[inter]**7
    b[inter] = 1.41338*y[inter] + 2.28305*y[inter]**2 + 1.07233*y[inter]**3 - 5.38434*y[inter]**4 - 0.62251*y[inter]**5 + 5.30260*y[inter]**6 - 2.09002*y[inter]**7
    # Infrared: 0.3 <= x <= 1.1 ; 9091 -> 33333 Angs ;
    inter = np.bitwise_and(x >= 0.3, x < 1.1)
    a[inter] =  0.574*x[inter]**1.61
    b[inter] = -0.527*x[inter]**1.61
    q_l = a + b/R_V
    return q_l

def Charlot_extlaw(l, mu=0.3):
    """
    Returns two-component dust model by Charlot and Fall 2000.

    Parameters
    ----------
    l : array like
        The \lambda wavelength array (in Angstroms).

    mu : float, optional
         Fraction of extinction contributed by the ambient ISM.
         Default value is 0.3.

    Returns
    -------
    array like
        \tau_lambda for t <= 10^7 yr.
    array like
        \tau_lambda for t > 10^7 yr.
    """

    t_lambda_Y = np.power((l/5500.), -0.7)
    t_lambda_O = mu*t_lambda_Y
    return t_lambda_Y, t_lambda_O

def Calzetti_extlaw(l, R_V=4.05):
    """
    Calculates the reddening law by Calzetti et al. (1994).
    Original comments in the .for file:

    q = A_lambda/A_V for Calzetti et al reddening law (formula from hyperz-manual).
    l = lambda, in Angstrons
    x = 1/lambda in 1/microns

    Parameters
    ----------
    l : array like
        Wavelength in Angstroms.
    R_V : float, optional
        Selective extinction parameter (roughly "slope").
        Default value is 4.05.

    Returns
    -------
    array like
        Extinction A_lamda/A_V. Array of same length as l.

    """
    if not isinstance(l, np.ma.MaskedArray):
        l = np.asarray(l, 'float64')
    x = 1e4/l
    q = np.zeros_like(l)
    # UV -> Optical: 1200 -> 6300 Angs
    inter = (l >= 1200.) & (l <= 6300.)
    q[inter] = (2.659/R_V)*(-2.156 + 1.509*x[inter] - 0.198*x[inter]**2 + 0.011*x[inter]**3) + 1.
    # Red -> Infrared
    inter = (l >= 6300.) & (l <= 22000.)
    q[inter] = (2.659/R_V)*(-1.857 + 1.040*x[inter]) + 1.
    # Issue a warning if lambda falls outside 1200->22000 Angs range
    if ((l < 1200.) | (l > 22000.)).any():
        print('[Calzetti_extlaw] WARNING! some lambda outside valid range (1200, 22000.)')
    return q

def calc_extlaw(l, R_V, extlaw=None):
    """
        A wrapper to select which extintion law to use if needed by user.

    Parameters
    ----------
    l : array like
        Wavelength in Angstroms.

    R_V : float
        Selective extinction parameter (roughly "slope").

    extlaw : str {'CCM', 'CAL'}, optional
        Which extinction function to use.
        CCM will call `Cardelli_extlaw`.
        CAL will call `Calzetti_extlaw`.
        Default value is CCM.

    Returns
    -------
    array like
        q(lambda) of the `extlaw` extinction law.
    """
    if (extlaw == 'CCM'):
        return Cardelli_extlaw(l, R_V)
    elif (extlaw == 'CAL'):
        return Calzetti_extlaw(l, R_V)
    else:
        raise ValueError((
            f'[{__name__}]: extlaw {extlaw} not inexistent. Try'
            f'CCM (Cardelli extinction law) or'
            f'CAL (Calzetti extinction law).'
        ))

def spec_apply_dust(wave, flux, AV, R_V=None, extlaw=None):
    """
    Apply the dust extinction law factor to a spectrum `flux` at wavelenghts
    `wave`.

        flux_intrinsic = `flux` * 10^(-0.4 * `AV` * q_l)

    Parameters
    ----------
    wave : array like
        Wavelenghts at restframe.

            (1 + redshift) = `wave` / wave_at_restframe

    flux : array like
        Fluxes to be dust-corrected.

    AV : float or array like
        Dust extinction in mag. If AV is an array, will create
        an (N_AV, N_wave) array of dust spectra.

    R_V : float or None
        Selective extinction parameter (roughly "slope"). Default value 3.1.

    extlaw : str {'CCM', 'CAL'} or None
        Which extinction function to use.
        CCM will call `Cardelli_extlaw`.
        CAL will call `Calzetti_extlaw`.
        Default value is CCM.

    Returns
    -------
    array like
        The `flux` correct by dust extinction.
    """
    if R_V is None:
        R_V = 3.1
    if extlaw is None:
        extlaw = 'CCM'
    q_l = calc_extlaw(wave, R_V, extlaw)
    dust_factor = 10**(-0.4*AV*q_l)
    return flux*dust_factor



#!/usr/bin/env python3
################################################################################
#
# flux_elines_cube_EW.py
# ----------------------
# Perl script: flux_elines_cube_EW.pl
#
################################################################################

### Local imports
import sys
import numpy as np
from astropy.io import fits
from os.path import basename
from copy import deepcopy as copy

from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.io import get_data_from_fits, array_to_fits
from pyFIT3D.common.constants import __FWHM_to_sigma__
from pyFIT3D.common.tools import flux_elines_cube_EW

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['cube_fits', 'emission_lines_txt', 'output_fits',
                     'vel_map_fits',  'FWHM_map_sigma_inst']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'cube_fits': str, 'emission_lines_txt': str,
                               'output_fits': str, 'vel_map_fits': str,
                               'FWHM_map_sigma_inst': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' CUBE.fits,e_CUBE.fits,N_MC,CONT_CUBE.fits'
    usage_msg_tmp3 = ' emission_lines.txt output.fits VEL_map.fits'
    usage_msg_tmp4 = ' FWHM_map.fits[sigma_inst]'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2 + usage_msg_tmp3 + \
        usage_msg_tmp4

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        self._parse_filenames()

    def _parse_filenames(self):
        self.e_cube_fits = None
        self.n_mc = 1
        self.cont_cube_fits = None
        self.min_flux = None # Check if it is used.
        cube_fits_args = self.cube_fits.split(',')
        # print(cube_fits_args)
        if "," in self.cube_fits:
            self.cube_fits = cube_fits_args[0]
            self.e_cube_fits = cube_fits_args[1]
            self.n_mc = int(cube_fits_args[2])
            self.cont_cube_fits = cube_fits_args[3]

def main(args):
    input_e_cube = args.e_cube_fits
    input_cont_cube = args.cont_cube_fits
    guided_sigma = args.FWHM_map_sigma_inst
    input_cube_data, input_cube_header = get_data_from_fits(args.cube_fits, header=True)
    if input_e_cube is not None:
        # print("Reading Error cube")
        if "[1]" in input_e_cube:
            input_e_cube=input_e_cube.replace('[1]','')
        e_cube = fits.open(input_e_cube)
        if len(e_cube) > 1:
            input_e_cube_data = copy(e_cube[1].data)
        else:
            input_e_cube_data = copy(e_cube[0].data)
    if input_cont_cube is not None:
        # print("Reading cont cube")
        input_cont_cube_data = get_data_from_fits(input_cont_cube)
    gmap = get_data_from_fits(args.vel_map_fits)
    if "fit" in guided_sigma:
        # print("guided_sigma is a file")
        s_gmap = get_data_from_fits(guided_sigma)*__FWHM_to_sigma__
    else:
        s_gmap = float(guided_sigma)
    out, input_cube_header = flux_elines_cube_EW(input_cube_data, input_cube_header,
                                                 args.n_mc, args.emission_lines_txt, gmap, s_gmap,
                                                 input_e_cube_data, input_cont_cube_data)
    array_to_fits(args.output_fits, out, header=input_cube_header,
                  overwrite=True, sort_dict_header=False)

if __name__ == '__main__':
    main(ReadArgumentsLocal())
    # parsed_args = ReadArgumentsLocal()
    # input_cube = parsed_args.cube_fits
    # input_e_cube = parsed_args.e_cube_fits
    # n_mc = int(parsed_args.n_mc)
    # input_cont_cube = parsed_args.cont_cube_fits
    # elines_list = parsed_args.emission_lines_txt
    # guided_map = parsed_args.vel_map_fits
    # guided_sigma = parsed_args.FWHM_map_sigma_inst
    # output = parsed_args.output_fits
    # flux_elines_cube_EW(input_cube, input_e_cube, n_mc, input_cont_cube, elines_list,
    #                     guided_map, guided_sigma, output)



#!/usr/bin/env python3
################################################################################
#
# write_img_header.py
# ----------------------
# Perl script: write_img_header.pl
#
################################################################################

# System imports
import sys
import warnings
from os.path import basename
from astropy.io.fits.verify import VerifyWarning

# Local imports
from pyFIT3D.common.io import ReadArguments, write_img_header

warnings.simplefilter('ignore', category=VerifyWarning)

def parse_value(x):
    try:
        _x = eval(x)
    except (NameError, SyntaxError):
        _x = str(x)
    return _x

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_cube_fits', 'keyword', 'value']
    __optional__ = ['comment', 'before', 'after', 'output_fits']
    __def_optional__ = {}
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_cube_fits': str, 'keyword': str, 'value': parse_value}
    __conv_func_optional__ = {'comment': str, 'before': str, 'after': str, 'output_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' input_cube.fits KEYWORD VALUE [COMMENT] [BEFORE] [AFTER] [OUTPUT_FITS]'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

if __name__ == '__main__':
    parsed_args = ReadArgumentsLocal()
    write_img_header(parsed_args.input_cube_fits, parsed_args.keyword, parsed_args.value,
                     comment=parsed_args.comment,
                     before=parsed_args.before,
                     after=parsed_args.after,
                     output=parsed_args.output_fits,
                     overwrite=True)



#!/usr/bin/env python3
################################################################################
#
# map_auto_ssp_rnd.seg.py
# ----------------------
# Perl script: map_auto_ssp_rnd.seg.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import map_auto_ssp_rnd_seg

class ReadArgumentsLocal(ReadArguments):
    """
    Local argument parser
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input', 'segmentation_fits', 'output_prefix']
    __optional__ = ['inst_disp', 'central_wavelength']
    __def_optional__ = {'central_wavelength': 4275, 'inst_disp': 2.45}
    __arg_names__ = __mandatory__  + __optional__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input': str, 'segmentation_fits': str, 'output_prefix': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    __usage_msg__ = f'USE: {__script_name__}'
    __usage_msg__ += ' elines_OUT seg_file.fits PREFIX_OUT [DISP_INST] [WAVE_NORM]'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)

if __name__ == '__main__':
    pa = ReadArgumentsLocal()
    map_auto_ssp_rnd_seg(pa.input, pa.segmentation_fits, pa.output_prefix,
                         pa.inst_disp, pa.central_wavelength)  #, pa.auto_ssp_config)



#!/usr/bin/env python3
################################################################################
#
# med2df.py
# ----------------------
# Perl script: med2df.pl
#
################################################################################

# System imports
import sys
from os.path import basename

# Local imports
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.tools import med2df

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for this script
    """
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['input_fits', 'output_fits', 'x_width', 'y_width']
    __arg_names__ = __mandatory__
    __N_tot_args__ = len(__arg_names__)
    __conv_func_mandatory__ = {'input_fits': str, 'output_fits': str}
    __conv_func__ = __conv_func_mandatory__.copy()

    usage_msg_tmp1 = 'USE: {}'.format(__script_name__)
    usage_msg_tmp2 = ' INPUT.FITS OUTPUT.FITS X_WIDTH Y_WIDTH'
    __usage_msg__ = usage_msg_tmp1 + usage_msg_tmp2

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        # self._parse_filenames()

if __name__ == '__main__':
    # pa stands for parsed arguments
    pa = ReadArgumentsLocal()
    med2df(pa.input_fits, pa.output_fits, pa.x_width, pa.y_width)



#!/usr/bin/env python3
#############################################################################
#
# Python version of auto_ssp_elines_rnd_rss_sigma_int_disp_min.pl
# -----------------------
# 2019-10-10: Eduardo A. D. Lacerda: Initial commit.
#
#############################################################################
import sys
import numpy as np
from os.path import basename
from pyFIT3D.common.io import ReadArguments
from pyFIT3D.common.auto_ssp_tools import auto_ssp_elines_rnd_rss

class ReadArgumentsLocal(ReadArguments):
    """
    Argument parser for the SSP models fit scripts.

    To add an argument to the script:
        Argument name in `__mandatory__` or `__optional__` list.
        Argument conversion function in `__mandatory_conv_func___` or `__optional_conv_func___` list.
        Argument default value (if not mandatory) in `__def_optional__`
    """
    # class static configuration:
    # arguments names and conversion string to number functions
    __script_name__ = basename(sys.argv[0])
    __mandatory__ = ['spec_file', 'ssp_files', 'out_file', 'mask_list', 'config_file', 'plot']
    __optional__ = ['min', 'max', 'w_min', 'w_max', 'elines_mask_file', 'input_redshift', 'delta_redshift', 'min_redshift', 'max_redshift', 'input_sigma', 'delta_sigma', 'min_sigma', 'max_sigma', 'input_AV', 'delta_AV', 'min_AV', 'max_AV']
    __arg_names__ = __mandatory__ + __optional__
    __N_tot_args__ = len(__arg_names__)
    # default values of optional arguments following __optional__ order
    __def_optional__ = {'input_redshift': 0}

    # parse functions
    __conv_func_mandatory__ = {'spec_file': str, 'ssp_files': str, 'out_file': str, 'mask_list': str, 'config_file': str}
    __conv_func_optional__ = {'elines_mask_file': str}
    __conv_func__ = __conv_func_mandatory__.copy()
    __conv_func__.update(__conv_func_optional__)

    # usage message
    __usage_msg__ = 'USE: {} SPEC1.RSS.fits[,ERROR.RSS.fits] SSP_SFH.fits,SSP_KIN.fits,sigma_inst OUTFILE MASK_LIST CONFIG_FILE PLOT'.format(__script_name__)
    __usage_msg__ += ' [min max] [wmin[,wmin2]] [wmax[,wmax2]] [redshift_elines_to_mask] [input_redshift delta_redshift min_redshift max_redshift]'
    __usage_msg__ += ' [input_sigma delta_sigma min_sigma max_sigma] [input_AV delta_AV min_AV max_AV] \n'
    __usage_msg__ += 'CONFIG_FILE:\n'
    __usage_msg__ += 'redshift delta_redshift min_redshift max_redshift\n'
    __usage_msg__ += 'sigma delta_sigma min_sigma max_sigma (km/s)\n'
    __usage_msg__ += 'AV delta_AV min_AV max_AV [Same range for all - magnitudes ]\n'
    __usage_msg__ += 'N_SYSTEMS\n'
    __usage_msg__ += '(1) START_W END_W MASK_FILE CONFIG_FILE NPOLY MASK_FILE_POLY\n'
    __usage_msg__ += '...\n'
    __usage_msg__ += '(N) START_W END_W MASK_FILE CONFIG_FILE NPOLY MASK_FILE_POLY\n'
    __usage_msg__ += 'MIN_DELTA_CHISQ MAX_NITER CUT_MEDIAN_FLUX\n'
    __usage_msg__ += 'start_w_peak end_w_peak\n'
    __usage_msg__ += 'wavelength_to_norm width_AA new_back_templates.fits\n'

    def __init__(self, args_list=None, verbose=False):
        ReadArguments.__init__(self, args_list, verbose=verbose)
        self._parse()

    def _parse(self):
        self.error_file = None
        spec_file_args = self.spec_file.split(',')
        if len(spec_file_args) == 2:
            self.spec_file = spec_file_args[0]
            self.error_file = spec_file_args[1]
        print(f'ef={self.error_file} sp={self.spec_file}')
        if self.w_min is None:
            self.w_min = -np.inf
        if self.w_max is None:
            self.w_max = np.inf
        self.nl_w_min = self.w_min
        self.nl_w_max = self.w_max
        if isinstance(self.w_min, tuple):
            w_min = self.w_min[0]
            nl_w_min = self.w_min[1]
            self.nl_w_min = nl_w_min
            self.w_min = w_min
        if isinstance(self.w_max, tuple):
            w_max = self.w_max[0]
            nl_w_max = self.w_max[1]
            self.nl_w_max = nl_w_max
            self.w_max = w_max
        self._parse_ssp_sigma_inst()
        self._final_check()

    def _final_check(self):
        self.redef = 0
        if self.min is not None and self.max is not None:
            self.redef = 1
        if self.w_min is not None and self.w_max is not None:
            self.redef = 2
        self.redshift_set = None
        if ((self.input_redshift is not None) and (self.delta_redshift is not None)
            and (self.min_redshift is not None) and (self.max_redshift is not None)):
            self.redshift_set = [self.input_redshift, self.delta_redshift,
                                 self.min_redshift, self.max_redshift]
        self.sigma_set = None
        if ((self.input_sigma is not None) and (self.delta_sigma is not None)
            and (self.min_sigma is not None) and (self.max_sigma is not None)):
            self.sigma_set = [self.input_sigma, self.delta_sigma,
                              self.min_sigma, self.max_sigma]
        self.AV_set = None
        if ((self.input_AV is not None) and (self.delta_AV is not None)
            and (self.min_AV is not None) and (self.max_AV is not None)):
            self.AV_set = [self.input_AV, self.delta_AV,
                              self.min_AV, self.max_AV]

    def _parse_ssp_sigma_inst(self):
        # parse SSP_SFH.fits,SSP_KIN.fits,sigma_inst argument
        ssp_args = self.ssp_files.split(',')
        n = len(ssp_args)
        self.ssp_file = ssp_args[0]
        self.ssp_nl_fit_file = ssp_args[0]
        self.sigma_inst = 0.01
        if n > 1:
            self.ssp_nl_fit_file = ssp_args[1]
        if n > 2:
            self.sigma_inst = np.float(ssp_args[2])
        if self.sigma_inst == 0:
            self.sigma_inst == 1
        print(f'sspnlf={self.ssp_nl_fit_file} sspf={self.ssp_file}')

if __name__ == '__main__':
    pa = ReadArgumentsLocal()
    auto_ssp_elines_rnd_rss(
        spec_file=pa.spec_file,
        error_file=pa.error_file,
        ssp_file=pa.ssp_file,
        ssp_nl_fit_file=pa.ssp_nl_fit_file,
        out_file=pa.out_file,
        config_file=pa.config_file,
        mask_list=pa.mask_list,
        elines_mask_file=pa.elines_mask_file,
        sigma_inst=pa.sigma_inst,
        min=pa.min, max=pa.max,
        w_min=pa.w_min, w_max=pa.w_max,
        nl_w_min=pa.nl_w_min, nl_w_max=pa.nl_w_max,
        plot=pa.plot,
        input_redshift=pa.input_redshift, delta_redshift=pa.delta_redshift, min_redshift=pa.min_redshift, max_redshift=pa.max_redshift,
        input_sigma=pa.input_sigma, delta_sigma=pa.delta_sigma, min_sigma=pa.min_sigma, max_sigma=pa.max_sigma,
        input_AV=pa.input_AV, delta_AV=pa.delta_AV, min_AV=pa.min_AV, max_AV=pa.max_AV,
        is_guided_sigma=True
    )
